@@ -25,31 +25,20 @@ const (
 	testTypeArchive = "archive"
 )
 
-// loadedTest contains all of data, details and dependencies of a fully-loaded
-// and configured k6 test.
+// loadedTest contains all of data, details and dependencies of a loaded
+// k6 test, but without any config consolidation.
 type loadedTest struct {
-	sourceRootPath  string // contains the raw string the user supplied
-	pwd             string
-	source          *loader.SourceData
-	fs              afero.Fs
-	fileSystems     map[string]afero.Fs
-	runtimeOptions  lib.RuntimeOptions
-	metricsRegistry *metrics.Registry
-	builtInMetrics  *metrics.BuiltinMetrics
-	initRunner      lib.Runner // TODO: rename to something more appropriate
-	keywriter       io.Closer
-
-	// Only set if cliConfigGetter is supplied to loadAndConfigureTest() or if
-	// consolidateDeriveAndValidateConfig() is manually called.
-	consolidatedConfig Config
-	derivedConfig      Config
+	sourceRootPath string // contains the raw string the user supplied
+	pwd            string
+	source         *loader.SourceData
+	fs             afero.Fs
+	fileSystems    map[string]afero.Fs
+	preInitState   *lib.TestPreInitState
+	initRunner     lib.Runner // TODO: rename to something more appropriate
+	keyLogger      io.Closer
 }
 
-func loadAndConfigureTest(
-	gs *globalState, cmd *cobra.Command, args []string,
-	// supply this if you want the test config consolidated and validated
-	cliConfigGetter func(flags *pflag.FlagSet) (Config, error), // TODO: obviate
-) (*loadedTest, error) {
+func loadTest(gs *globalState, cmd *cobra.Command, args []string) (*loadedTest, error) {
 	if len(args) < 1 {
 		return nil, fmt.Errorf("k6 needs at least one argument to load the test")
 	}
@@ -73,15 +62,20 @@ func loadAndConfigureTest(
 	}
 
 	registry := metrics.NewRegistry()
+	state := &lib.TestPreInitState{
+		Logger:         gs.logger,
+		RuntimeOptions: runtimeOptions,
+		Registry:       registry,
+		BuiltinMetrics: metrics.RegisterBuiltinMetrics(registry),
+	}
+
 	test := &loadedTest{
-		pwd:             pwd,
-		sourceRootPath:  sourceRootPath,
-		source:          src,
-		fs:              gs.fs,
-		fileSystems:     fileSystems,
-		runtimeOptions:  runtimeOptions,
-		metricsRegistry: registry,
-		builtInMetrics:  metrics.RegisterBuiltinMetrics(registry),
+		pwd:            pwd,
+		sourceRootPath: sourceRootPath,
+		source:         src,
+		fs:             gs.fs,
+		fileSystems:    fileSystems,
+		preInitState:   state,
 	}
 
 	gs.logger.Debugf("Initializing k6 runner for '%s' (%s)...", sourceRootPath, resolvedPath)
@@ -89,13 +83,6 @@ func loadAndConfigureTest(
 		return nil, fmt.Errorf("could not initialize '%s': %w", sourceRootPath, err)
 	}
 	gs.logger.Debug("Runner successfully initialized!")
-
-	if cliConfigGetter != nil {
-		if err := test.consolidateDeriveAndValidateConfig(gs, cmd, cliConfigGetter); err != nil {
-			return nil, err
-		}
-	}
-
 	return test, nil
 }
 
@@ -103,33 +90,33 @@ func (lt *loadedTest) initializeFirstRunner(gs *globalState) error {
 	testPath := lt.source.URL.String()
 	logger := gs.logger.WithField("test_path", testPath)
 
-	testType := lt.runtimeOptions.TestType.String
+	testType := lt.preInitState.RuntimeOptions.TestType.String
 	if testType == "" {
 		logger.Debug("Detecting test type for...")
 		testType = detectTestType(lt.source.Data)
 	}
 
-	state := &lib.RuntimeState{
-		Logger:         gs.logger,
-		RuntimeOptions: lt.runtimeOptions,
-		BuiltinMetrics: lt.builtInMetrics,
-		Registry:       lt.metricsRegistry,
-	}
-	if lt.runtimeOptions.KeyWriter.Valid {
+	if lt.preInitState.RuntimeOptions.KeyWriter.Valid {
 		logger.Warnf("SSLKEYLOGFILE was specified, logging TLS connection keys to '%s'...",
-			lt.runtimeOptions.KeyWriter.String)
-		keyfileName := filepath.Join(lt.pwd, lt.runtimeOptions.KeyWriter.String)
-		f, err := lt.fs.OpenFile(keyfileName, os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0o600)
-		if err != nil {
-			return err
+			lt.preInitState.RuntimeOptions.KeyWriter.String)
+		keylogFilename := lt.preInitState.RuntimeOptions.KeyWriter.String
+		// if path is absolute - no point doing anything
+		if !filepath.IsAbs(keylogFilename) {
+			// filepath.Abs could be used but it will get the pwd from `os` package instead of what is in lt.pwd
+			// this is against our general approach of not using `os` directly and makes testing harder
+			keylogFilename = filepath.Join(lt.pwd, keylogFilename)
 		}
-		lt.keywriter = f
-		state.KeyLogger = &syncWriter{w: f}
+		f, err := lt.fs.OpenFile(keylogFilename, os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0o600)
+		if err != nil {
+			return fmt.Errorf("couldn't get absolute path for keylog file: %w", err)
+		}
+		lt.keyLogger = f
+		lt.preInitState.KeyLogger = &syncWriter{w: f}
 	}
 	switch testType {
 	case testTypeJS:
 		logger.Debug("Trying to load as a JS test...")
-		runner, err := js.New(state, lt.source, lt.fileSystems)
+		runner, err := js.New(lt.preInitState, lt.source, lt.fileSystems)
 		// TODO: should we use common.UnwrapGojaInterruptedError() here?
 		if err != nil {
 			return fmt.Errorf("could not load JS test '%s': %w", testPath, err)
@@ -150,7 +137,7 @@ func (lt *loadedTest) initializeFirstRunner(gs *globalState) error {
 		switch arc.Type {
 		case testTypeJS:
 			logger.Debug("Evaluating JS from archive bundle...")
-			lt.initRunner, err = js.NewFromArchive(state, arc)
+			lt.initRunner, err = js.NewFromArchive(lt.preInitState, arc)
 			if err != nil {
 				return fmt.Errorf("could not load JS from test archive bundle '%s': %w", testPath, err)
 			}
@@ -186,50 +173,88 @@ func detectTestType(data []byte) string {
 func (lt *loadedTest) consolidateDeriveAndValidateConfig(
 	gs *globalState, cmd *cobra.Command,
 	cliConfGetter func(flags *pflag.FlagSet) (Config, error), // TODO: obviate
-) error {
+) (*loadedAndConfiguredTest, error) {
 	var cliConfig Config
 	if cliConfGetter != nil {
 		gs.logger.Debug("Parsing CLI flags...")
 		var err error
 		cliConfig, err = cliConfGetter(cmd.Flags())
 		if err != nil {
-			return err
+			return nil, err
 		}
 	}
 
 	gs.logger.Debug("Consolidating config layers...")
 	consolidatedConfig, err := getConsolidatedConfig(gs, cliConfig, lt.initRunner.GetOptions())
 	if err != nil {
-		return err
+		return nil, err
 	}
 
 	gs.logger.Debug("Parsing thresholds and validating config...")
 	// Parse the thresholds, only if the --no-threshold flag is not set.
 	// If parsing the threshold expressions failed, consider it as an
 	// invalid configuration error.
-	if !lt.runtimeOptions.NoThresholds.Bool {
+	if !lt.preInitState.RuntimeOptions.NoThresholds.Bool {
 		for metricName, thresholdsDefinition := range consolidatedConfig.Options.Thresholds {
 			err = thresholdsDefinition.Parse()
 			if err != nil {
-				return errext.WithExitCodeIfNone(err, exitcodes.InvalidConfig)
+				return nil, errext.WithExitCodeIfNone(err, exitcodes.InvalidConfig)
 			}
 
-			err = thresholdsDefinition.Validate(metricName, lt.metricsRegistry)
+			err = thresholdsDefinition.Validate(metricName, lt.preInitState.Registry)
 			if err != nil {
-				return errext.WithExitCodeIfNone(err, exitcodes.InvalidConfig)
+				return nil, errext.WithExitCodeIfNone(err, exitcodes.InvalidConfig)
 			}
 		}
 	}
 
 	derivedConfig, err := deriveAndValidateConfig(consolidatedConfig, lt.initRunner.IsExecutable, gs.logger)
 	if err != nil {
-		return err
+		return nil, err
 	}
 
-	lt.consolidatedConfig = consolidatedConfig
-	lt.derivedConfig = derivedConfig
+	return &loadedAndConfiguredTest{
+		loadedTest:         lt,
+		consolidatedConfig: consolidatedConfig,
+		derivedConfig:      derivedConfig,
+	}, nil
+}
 
-	return nil
+// loadedAndConfiguredTest contains the whole loadedTest, as well as the
+// consolidated test config and the full test run state.
+type loadedAndConfiguredTest struct {
+	*loadedTest
+	consolidatedConfig Config
+	derivedConfig      Config
+}
+
+func loadAndConfigureTest(
+	gs *globalState, cmd *cobra.Command, args []string,
+	cliConfigGetter func(flags *pflag.FlagSet) (Config, error),
+) (*loadedAndConfiguredTest, error) {
+	test, err := loadTest(gs, cmd, args)
+	if err != nil {
+		return nil, err
+	}
+
+	return test.consolidateDeriveAndValidateConfig(gs, cmd, cliConfigGetter)
+}
+
+func (lct *loadedAndConfiguredTest) buildTestRunState(
+	configToReinject lib.Options,
+) (*lib.TestRunState, error) {
+	// This might be the full derived or just the consodlidated options
+	if err := lct.initRunner.SetOptions(configToReinject); err != nil {
+		return nil, err
+	}
+
+	// TODO: init atlas root node, etc.
+
+	return &lib.TestRunState{
+		TestPreInitState: lct.preInitState,
+		Runner:           lct.initRunner,
+		Options:          lct.derivedConfig.Options, // we will always run with the derived options
+	}, nil
 }
 
 type syncWriter struct {
