@@ -1,23 +1,3 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2016 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
-
 package core
 
 import (
@@ -58,32 +38,51 @@ type testStruct struct {
 	run       func() error
 	runCancel func()
 	wait      func()
+	piState   *lib.TestPreInitState
+}
+
+func getTestPreInitState(tb testing.TB) *lib.TestPreInitState {
+	reg := metrics.NewRegistry()
+	return &lib.TestPreInitState{
+		Logger:         testutils.NewLogger(tb),
+		RuntimeOptions: lib.RuntimeOptions{},
+		Registry:       reg,
+		BuiltinMetrics: metrics.RegisterBuiltinMetrics(reg),
+	}
+}
+
+func getTestRunState(
+	tb testing.TB, piState *lib.TestPreInitState, options lib.Options, runner lib.Runner,
+) *lib.TestRunState {
+	require.Empty(tb, options.Validate())
+	require.NoError(tb, runner.SetOptions(options))
+	return &lib.TestRunState{
+		TestPreInitState: piState,
+		Options:          options,
+		Runner:           runner,
+	}
 }
 
 // Wrapper around NewEngine that applies a logger and manages the options.
-func newTestEngineWithRegistry( //nolint:golint
-	t *testing.T, runTimeout *time.Duration, runner lib.Runner, outputs []output.Output, opts lib.Options,
-	registry *metrics.Registry,
+func newTestEngineWithTestPreInitState( //nolint:golint
+	t *testing.T, runTimeout *time.Duration, runner lib.Runner, outputs []output.Output,
+	opts lib.Options, piState *lib.TestPreInitState,
 ) *testStruct {
 	if runner == nil {
 		runner = &minirunner.MiniRunner{}
 	}
 
-	logger := logrus.New()
-	logger.SetOutput(testutils.NewTestOutput(t))
 	newOpts, err := executor.DeriveScenariosFromShortcuts(lib.Options{
 		MetricSamplesBufferSize: null.NewInt(200, false),
-	}.Apply(runner.GetOptions()).Apply(opts), logger)
-	require.NoError(t, err)
-	require.Empty(t, newOpts.Validate())
-
-	require.NoError(t, runner.SetOptions(newOpts))
-
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
-	execScheduler, err := local.NewExecutionScheduler(runner, builtinMetrics, logger)
+	}.Apply(runner.GetOptions()).Apply(opts), piState.Logger)
 	require.NoError(t, err)
 
-	engine, err := NewEngine(execScheduler, opts, lib.RuntimeOptions{}, outputs, logger, registry)
+	testRunState := getTestRunState(t, piState, newOpts, runner)
+
+	execScheduler, err := local.NewExecutionScheduler(testRunState)
+	require.NoError(t, err)
+
+	engine, err := NewEngine(testRunState, execScheduler, outputs)
 	require.NoError(t, err)
 	require.NoError(t, engine.OutputManager.StartOutputs())
 
@@ -109,6 +108,7 @@ func newTestEngineWithRegistry( //nolint:golint
 			waitFn()
 			engine.OutputManager.StopOutputs()
 		},
+		piState: piState,
 	}
 	return test
 }
@@ -116,7 +116,7 @@ func newTestEngineWithRegistry( //nolint:golint
 func newTestEngine(
 	t *testing.T, runTimeout *time.Duration, runner lib.Runner, outputs []output.Output, opts lib.Options,
 ) *testStruct {
-	return newTestEngineWithRegistry(t, runTimeout, runner, outputs, opts, metrics.NewRegistry())
+	return newTestEngineWithTestPreInitState(t, runTimeout, runner, outputs, opts, getTestPreInitState(t))
 }
 
 func TestEngineRun(t *testing.T) {
@@ -156,8 +156,8 @@ func TestEngineRun(t *testing.T) {
 	t.Run("collects samples", func(t *testing.T) {
 		t.Parallel()
 
-		registry := metrics.NewRegistry()
-		testMetric, err := registry.NewMetric("test_metric", metrics.Trend)
+		piState := getTestPreInitState(t)
+		testMetric, err := piState.Registry.NewMetric("test_metric", metrics.Trend)
 		require.NoError(t, err)
 
 		signalChan := make(chan interface{})
@@ -173,10 +173,10 @@ func TestEngineRun(t *testing.T) {
 		}
 
 		mockOutput := mockoutput.New()
-		test := newTestEngineWithRegistry(t, nil, runner, []output.Output{mockOutput}, lib.Options{
+		test := newTestEngineWithTestPreInitState(t, nil, runner, []output.Output{mockOutput}, lib.Options{
 			VUs:        null.IntFrom(1),
 			Iterations: null.IntFrom(1),
-		}, registry)
+		}, piState)
 
 		errC := make(chan error)
 		go func() { errC <- test.run() }()
@@ -226,8 +226,8 @@ func TestEngineStopped(t *testing.T) {
 func TestEngineOutput(t *testing.T) {
 	t.Parallel()
 
-	registry := metrics.NewRegistry()
-	testMetric, err := registry.NewMetric("test_metric", metrics.Trend)
+	piState := getTestPreInitState(t)
+	testMetric, err := piState.Registry.NewMetric("test_metric", metrics.Trend)
 	require.NoError(t, err)
 
 	runner := &minirunner.MiniRunner{
@@ -238,10 +238,10 @@ func TestEngineOutput(t *testing.T) {
 	}
 
 	mockOutput := mockoutput.New()
-	test := newTestEngineWithRegistry(t, nil, runner, []output.Output{mockOutput}, lib.Options{
+	test := newTestEngineWithTestPreInitState(t, nil, runner, []output.Output{mockOutput}, lib.Options{
 		VUs:        null.IntFrom(1),
 		Iterations: null.IntFrom(1),
-	}, registry)
+	}, piState)
 
 	assert.NoError(t, test.run())
 	test.wait()
@@ -254,7 +254,7 @@ func TestEngineOutput(t *testing.T) {
 	}
 	metric := test.engine.MetricsEngine.ObservedMetrics["test_metric"]
 	if assert.NotNil(t, metric) {
-		sink := metric.Sink.(*metrics.TrendSink) // nolint: forcetypeassert
+		sink := metric.Sink.(*metrics.TrendSink) //nolint:forcetypeassert
 		if assert.NotNil(t, sink) {
 			numOutputSamples := len(cSamples)
 			numEngineSamples := len(sink.Values)
@@ -269,17 +269,31 @@ func TestEngine_processSamples(t *testing.T) {
 	t.Run("metric", func(t *testing.T) {
 		t.Parallel()
 
-		registry := metrics.NewRegistry()
-		metric, err := registry.NewMetric("my_metric", metrics.Gauge)
+		piState := getTestPreInitState(t)
+		metric, err := piState.Registry.NewMetric("my_metric", metrics.Gauge)
 		require.NoError(t, err)
 
-		test := newTestEngineWithRegistry(t, nil, nil, nil, lib.Options{}, registry)
+		done := make(chan struct{})
+		runner := &minirunner.MiniRunner{
+			Fn: func(ctx context.Context, _ *lib.State, out chan<- metrics.SampleContainer) error {
+				out <- metrics.Sample{Metric: metric, Value: 1.25, Tags: metrics.IntoSampleTags(&map[string]string{"a": "1"})}
+				close(done)
+				return nil
+			},
+		}
+		test := newTestEngineWithTestPreInitState(t, nil, runner, nil, lib.Options{}, piState)
 
-		test.engine.OutputManager.AddMetricSamples(
-			[]metrics.SampleContainer{metrics.Sample{Metric: metric, Value: 1.25, Tags: metrics.IntoSampleTags(&map[string]string{"a": "1"})}},
-		)
+		go func() {
+			assert.NoError(t, test.run())
+		}()
+
+		select {
+		case <-done:
+			return
+		case <-time.After(10 * time.Second):
+			assert.Fail(t, "Test should have completed within 10 seconds")
+		}
 
-		test.engine.Stop()
 		test.wait()
 
 		assert.IsType(t, &metrics.GaugeSink{}, test.engine.MetricsEngine.ObservedMetrics["my_metric"].Sink)
@@ -287,25 +301,38 @@ func TestEngine_processSamples(t *testing.T) {
 	t.Run("submetric", func(t *testing.T) {
 		t.Parallel()
 
-		registry := metrics.NewRegistry()
-		metric, err := registry.NewMetric("my_metric", metrics.Gauge)
+		piState := getTestPreInitState(t)
+		metric, err := piState.Registry.NewMetric("my_metric", metrics.Gauge)
 		require.NoError(t, err)
 
 		ths := metrics.NewThresholds([]string{`value<2`})
 		gotParseErr := ths.Parse()
 		require.NoError(t, gotParseErr)
 
-		test := newTestEngineWithRegistry(t, nil, nil, nil, lib.Options{
+		done := make(chan struct{})
+		runner := &minirunner.MiniRunner{
+			Fn: func(ctx context.Context, _ *lib.State, out chan<- metrics.SampleContainer) error {
+				out <- metrics.Sample{Metric: metric, Value: 1.25, Tags: metrics.IntoSampleTags(&map[string]string{"a": "1", "b": "2"})}
+				close(done)
+				return nil
+			},
+		}
+		test := newTestEngineWithTestPreInitState(t, nil, runner, nil, lib.Options{
 			Thresholds: map[string]metrics.Thresholds{
 				"my_metric{a:1}": ths,
 			},
-		}, registry)
+		}, piState)
 
-		test.engine.OutputManager.AddMetricSamples(
-			[]metrics.SampleContainer{metrics.Sample{Metric: metric, Value: 1.25, Tags: metrics.IntoSampleTags(&map[string]string{"a": "1", "b": "2"})}},
-		)
+		go func() {
+			assert.NoError(t, test.run())
+		}()
 
-		test.engine.Stop()
+		select {
+		case <-done:
+			return
+		case <-time.After(10 * time.Second):
+			assert.Fail(t, "Test should have completed within 10 seconds")
+		}
 		test.wait()
 
 		assert.Len(t, test.engine.MetricsEngine.ObservedMetrics, 2)
@@ -320,8 +347,8 @@ func TestEngine_processSamples(t *testing.T) {
 func TestEngineThresholdsWillAbort(t *testing.T) {
 	t.Parallel()
 
-	registry := metrics.NewRegistry()
-	metric, err := registry.NewMetric("my_metric", metrics.Gauge)
+	piState := getTestPreInitState(t)
+	metric, err := piState.Registry.NewMetric("my_metric", metrics.Gauge)
 	require.NoError(t, err)
 
 	// The incoming samples for the metric set it to 1.25. Considering
@@ -334,12 +361,26 @@ func TestEngineThresholdsWillAbort(t *testing.T) {
 
 	thresholds := map[string]metrics.Thresholds{metric.Name: ths}
 
-	test := newTestEngineWithRegistry(t, nil, nil, nil, lib.Options{Thresholds: thresholds}, registry)
+	done := make(chan struct{})
+	runner := &minirunner.MiniRunner{
+		Fn: func(ctx context.Context, _ *lib.State, out chan<- metrics.SampleContainer) error {
+			out <- metrics.Sample{Metric: metric, Value: 1.25, Tags: metrics.IntoSampleTags(&map[string]string{"a": "1"})}
+			close(done)
+			return nil
+		},
+	}
+	test := newTestEngineWithTestPreInitState(t, nil, runner, nil, lib.Options{Thresholds: thresholds}, piState)
 
-	test.engine.OutputManager.AddMetricSamples(
-		[]metrics.SampleContainer{metrics.Sample{Metric: metric, Value: 1.25, Tags: metrics.IntoSampleTags(&map[string]string{"a": "1"})}},
-	)
-	test.engine.Stop()
+	go func() {
+		assert.NoError(t, test.run())
+	}()
+
+	select {
+	case <-done:
+		return
+	case <-time.After(10 * time.Second):
+		assert.Fail(t, "Test should have completed within 10 seconds")
+	}
 	test.wait()
 	assert.True(t, test.engine.thresholdsTainted)
 }
@@ -347,8 +388,8 @@ func TestEngineThresholdsWillAbort(t *testing.T) {
 func TestEngineAbortedByThresholds(t *testing.T) {
 	t.Parallel()
 
-	registry := metrics.NewRegistry()
-	metric, err := registry.NewMetric("my_metric", metrics.Gauge)
+	piState := getTestPreInitState(t)
+	metric, err := piState.Registry.NewMetric("my_metric", metrics.Gauge)
 	require.NoError(t, err)
 
 	// The MiniRunner sets the value of the metric to 1.25. Considering
@@ -372,7 +413,7 @@ func TestEngineAbortedByThresholds(t *testing.T) {
 		},
 	}
 
-	test := newTestEngineWithRegistry(t, nil, runner, nil, lib.Options{Thresholds: thresholds}, registry)
+	test := newTestEngineWithTestPreInitState(t, nil, runner, nil, lib.Options{Thresholds: thresholds}, piState)
 	defer test.wait()
 
 	go func() {
@@ -422,12 +463,12 @@ func TestEngine_processThresholds(t *testing.T) {
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
-			registry := metrics.NewRegistry()
-			gaugeMetric, err := registry.NewMetric("my_metric", metrics.Gauge)
+			piState := getTestPreInitState(t)
+			gaugeMetric, err := piState.Registry.NewMetric("my_metric", metrics.Gauge)
 			require.NoError(t, err)
-			counterMetric, err := registry.NewMetric("used_counter", metrics.Counter)
+			counterMetric, err := piState.Registry.NewMetric("used_counter", metrics.Counter)
 			require.NoError(t, err)
-			_, err = registry.NewMetric("unused_counter", metrics.Counter)
+			_, err = piState.Registry.NewMetric("unused_counter", metrics.Counter)
 			require.NoError(t, err)
 
 			thresholds := make(map[string]metrics.Thresholds, len(data.ths))
@@ -439,8 +480,8 @@ func TestEngine_processThresholds(t *testing.T) {
 			}
 
 			runner := &minirunner.MiniRunner{}
-			test := newTestEngineWithRegistry(
-				t, nil, runner, nil, lib.Options{Thresholds: thresholds}, registry,
+			test := newTestEngineWithTestPreInitState(
+				t, nil, runner, nil, lib.Options{Thresholds: thresholds}, piState,
 			)
 
 			test.engine.OutputManager.AddMetricSamples(
@@ -550,14 +591,8 @@ func TestSentReceivedMetrics(t *testing.T) {
 	}
 
 	runTest := func(t *testing.T, ts testScript, tc testCase, noConnReuse bool) (float64, float64) {
-		registry := metrics.NewRegistry()
-		builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 		r, err := js.New(
-			&lib.RuntimeState{
-				Logger:         testutils.NewLogger(t),
-				BuiltinMetrics: builtinMetrics,
-				Registry:       registry,
-			},
+			getTestPreInitState(t),
 			&loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: []byte(ts.Code)},
 			nil,
 		)
@@ -635,8 +670,14 @@ func TestRunTags(t *testing.T) {
 	t.Parallel()
 	tb := httpmultibin.NewHTTPMultiBin(t)
 
-	runTagsMap := map[string]string{"foo": "bar", "test": "mest", "over": "written"}
-	runTags := metrics.NewSampleTags(runTagsMap)
+	expectedRunTags := map[string]string{"foo": "bar", "test": "mest", "over": "written"}
+
+	// it copies the map so in the case the runner will overwrite
+	// some run tags' values it doesn't affect the assertion.
+	runTags := make(map[string]string)
+	for k, v := range expectedRunTags {
+		runTags[k] = v
+	}
 
 	script := []byte(tb.Replacer.Replace(`
 		import http from "k6/http";
@@ -689,14 +730,8 @@ func TestRunTags(t *testing.T) {
 		}
 	`))
 
-	registry := metrics.NewRegistry()
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r, err := js.New(
-		&lib.RuntimeState{
-			Logger:         testutils.NewLogger(t),
-			BuiltinMetrics: builtinMetrics,
-			Registry:       registry,
-		},
+		getTestPreInitState(t),
 		&loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: script},
 		nil,
 	)
@@ -731,14 +766,14 @@ func TestRunTags(t *testing.T) {
 	getExpectedOverVal := func(metricName string) string {
 		for _, sysMetric := range systemMetrics {
 			if sysMetric == metricName {
-				return runTagsMap["over"]
+				return expectedRunTags["over"]
 			}
 		}
 		return "the rainbow"
 	}
 
 	for _, s := range mockOutput.Samples {
-		for key, expVal := range runTagsMap {
+		for key, expVal := range expectedRunTags {
 			val, ok := s.Tags.Get(key)
 
 			if key == "over" {
@@ -751,73 +786,6 @@ func TestRunTags(t *testing.T) {
 	}
 }
 
-func TestSetupTeardownThresholds(t *testing.T) {
-	t.Parallel()
-	tb := httpmultibin.NewHTTPMultiBin(t)
-
-	script := []byte(tb.Replacer.Replace(`
-		import http from "k6/http";
-		import { check } from "k6";
-		import { Counter } from "k6/metrics";
-
-		let statusCheck = { "status is 200": (r) => r.status === 200 }
-		let myCounter = new Counter("setup_teardown");
-
-		export let options = {
-			iterations: 5,
-			thresholds: {
-				"setup_teardown": ["count == 2"],
-				"iterations": ["count == 5"],
-				"http_reqs": ["count == 7"],
-			},
-		};
-
-		export function setup() {
-			check(http.get("HTTPBIN_IP_URL"), statusCheck) && myCounter.add(1);
-		};
-
-		export default function () {
-			check(http.get("HTTPBIN_IP_URL"), statusCheck);
-		};
-
-		export function teardown() {
-			check(http.get("HTTPBIN_IP_URL"), statusCheck) && myCounter.add(1);
-		};
-	`))
-
-	registry := metrics.NewRegistry()
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
-	runner, err := js.New(
-		&lib.RuntimeState{
-			Logger:         testutils.NewLogger(t),
-			BuiltinMetrics: builtinMetrics,
-			Registry:       registry,
-		},
-		&loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: script},
-		nil,
-	)
-	require.NoError(t, err)
-
-	test := newTestEngine(t, nil, runner, nil, lib.Options{
-		SystemTags:      &metrics.DefaultSystemTagSet,
-		SetupTimeout:    types.NullDurationFrom(3 * time.Second),
-		TeardownTimeout: types.NullDurationFrom(3 * time.Second),
-		VUs:             null.IntFrom(3),
-	})
-	defer test.wait()
-
-	errC := make(chan error)
-	go func() { errC <- test.run() }()
-
-	select {
-	case <-time.After(10 * time.Second):
-		t.Fatal("Test timed out")
-	case err := <-errC:
-		require.NoError(t, err)
-		require.False(t, test.engine.IsTainted())
-	}
-}
-
 func TestSetupException(t *testing.T) {
 	t.Parallel()
 
@@ -839,14 +807,8 @@ func TestSetupException(t *testing.T) {
 		        throw new Error("baz");
 			}
 	`), 0x666))
-	registry := metrics.NewRegistry()
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	runner, err := js.New(
-		&lib.RuntimeState{
-			Logger:         testutils.NewLogger(t),
-			BuiltinMetrics: builtinMetrics,
-			Registry:       registry,
-		},
+		getTestPreInitState(t),
 		&loader.SourceData{URL: &url.URL{Scheme: "file", Path: "/script.js"}, Data: script},
 		map[string]afero.Fs{"file": memfs},
 	)
@@ -892,15 +854,9 @@ func TestVuInitException(t *testing.T) {
 		}
 	`)
 
-	logger := testutils.NewLogger(t)
-	registry := metrics.NewRegistry()
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
+	piState := getTestPreInitState(t)
 	runner, err := js.New(
-		&lib.RuntimeState{
-			Logger:         testutils.NewLogger(t),
-			BuiltinMetrics: builtinMetrics,
-			Registry:       registry,
-		},
+		piState,
 		&loader.SourceData{URL: &url.URL{Scheme: "file", Path: "/script.js"}, Data: script},
 		nil,
 	)
@@ -908,12 +864,12 @@ func TestVuInitException(t *testing.T) {
 
 	opts, err := executor.DeriveScenariosFromShortcuts(runner.GetOptions(), nil)
 	require.NoError(t, err)
-	require.Empty(t, opts.Validate())
-	require.NoError(t, runner.SetOptions(opts))
 
-	execScheduler, err := local.NewExecutionScheduler(runner, builtinMetrics, logger)
+	testState := getTestRunState(t, piState, opts, runner)
+
+	execScheduler, err := local.NewExecutionScheduler(testState)
 	require.NoError(t, err)
-	engine, err := NewEngine(execScheduler, opts, lib.RuntimeOptions{}, nil, logger, registry)
+	engine, err := NewEngine(testState, execScheduler, nil)
 	require.NoError(t, err)
 
 	ctx, cancel := context.WithCancel(context.Background())
@@ -924,7 +880,7 @@ func TestVuInitException(t *testing.T) {
 
 	var exception errext.Exception
 	require.ErrorAs(t, err, &exception)
-	assert.Equal(t, "Error: oops in 2\n\tat file:///script.js:10:9(31)\n", err.Error())
+	assert.Equal(t, "Error: oops in 2\n\tat file:///script.js:10:9(29)\n\tat native\n", err.Error())
 
 	var errWithHint errext.HasHint
 	require.ErrorAs(t, err, &errWithHint)
@@ -966,14 +922,8 @@ func TestEmittedMetricsWhenScalingDown(t *testing.T) {
 		};
 	`))
 
-	registry := metrics.NewRegistry()
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	runner, err := js.New(
-		&lib.RuntimeState{
-			Logger:         testutils.NewLogger(t),
-			BuiltinMetrics: builtinMetrics,
-			Registry:       registry,
-		},
+		getTestPreInitState(t),
 		&loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: script},
 		nil,
 	)
@@ -1055,14 +1005,8 @@ func TestMetricsEmission(t *testing.T) {
 			if !isWindows {
 				t.Parallel()
 			}
-			registry := metrics.NewRegistry()
-			builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 			runner, err := js.New(
-				&lib.RuntimeState{
-					Logger:         testutils.NewLogger(t),
-					BuiltinMetrics: builtinMetrics,
-					Registry:       registry,
-				},
+				getTestPreInitState(t),
 				&loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: []byte(fmt.Sprintf(`
 				import { sleep } from "k6";
 				import { Counter } from "k6/metrics";
@@ -1110,7 +1054,6 @@ func TestMetricsEmission(t *testing.T) {
 	}
 }
 
-//nolint: funlen
 func TestMinIterationDurationInSetupTeardownStage(t *testing.T) {
 	t.Parallel()
 	setupScript := `
@@ -1168,14 +1111,8 @@ func TestMinIterationDurationInSetupTeardownStage(t *testing.T) {
 		tc := tc
 		t.Run(tc.name, func(t *testing.T) {
 			t.Parallel()
-			registry := metrics.NewRegistry()
-			builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 			runner, err := js.New(
-				&lib.RuntimeState{
-					Logger:         testutils.NewLogger(t),
-					BuiltinMetrics: builtinMetrics,
-					Registry:       registry,
-				},
+				getTestPreInitState(t),
 				&loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: []byte(tc.script)},
 				nil,
 			)
@@ -1200,8 +1137,8 @@ func TestMinIterationDurationInSetupTeardownStage(t *testing.T) {
 func TestEngineRunsTeardownEvenAfterTestRunIsAborted(t *testing.T) {
 	t.Parallel()
 
-	registry := metrics.NewRegistry()
-	testMetric, err := registry.NewMetric("teardown_metric", metrics.Counter)
+	piState := getTestPreInitState(t)
+	testMetric, err := piState.Registry.NewMetric("teardown_metric", metrics.Counter)
 	require.NoError(t, err)
 
 	var test *testStruct
@@ -1217,9 +1154,9 @@ func TestEngineRunsTeardownEvenAfterTestRunIsAborted(t *testing.T) {
 	}
 
 	mockOutput := mockoutput.New()
-	test = newTestEngineWithRegistry(t, nil, runner, []output.Output{mockOutput}, lib.Options{
+	test = newTestEngineWithTestPreInitState(t, nil, runner, []output.Output{mockOutput}, lib.Options{
 		VUs: null.IntFrom(1), Iterations: null.IntFrom(1),
-	}, registry)
+	}, piState)
 
 	assert.NoError(t, test.run())
 	test.wait()
@@ -1286,14 +1223,13 @@ func TestActiveVUsCount(t *testing.T) {
 	rtOpts := lib.RuntimeOptions{CompatibilityMode: null.StringFrom("base")}
 
 	registry := metrics.NewRegistry()
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
-	runner, err := js.New(
-		&lib.RuntimeState{
-			Logger:         logger,
-			BuiltinMetrics: builtinMetrics,
-			Registry:       registry,
-			RuntimeOptions: rtOpts,
-		}, &loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: script}, nil)
+	piState := &lib.TestPreInitState{
+		Logger:         logger,
+		Registry:       registry,
+		BuiltinMetrics: metrics.RegisterBuiltinMetrics(registry),
+		RuntimeOptions: rtOpts,
+	}
+	runner, err := js.New(piState, &loader.SourceData{URL: &url.URL{Path: "/script.js"}, Data: script}, nil)
 	require.NoError(t, err)
 
 	mockOutput := mockoutput.New()
@@ -1304,11 +1240,11 @@ func TestActiveVUsCount(t *testing.T) {
 		MetricSamplesBufferSize: null.NewInt(200, false),
 	}.Apply(runner.GetOptions()), nil)
 	require.NoError(t, err)
-	require.Empty(t, opts.Validate())
-	require.NoError(t, runner.SetOptions(opts))
-	execScheduler, err := local.NewExecutionScheduler(runner, builtinMetrics, logger)
+
+	testState := getTestRunState(t, piState, opts, runner)
+	execScheduler, err := local.NewExecutionScheduler(testState)
 	require.NoError(t, err)
-	engine, err := NewEngine(execScheduler, opts, rtOpts, []output.Output{mockOutput}, logger, registry)
+	engine, err := NewEngine(testState, execScheduler, []output.Output{mockOutput})
 	require.NoError(t, err)
 	require.NoError(t, engine.OutputManager.StartOutputs())
 	run, waitFn, err := engine.Init(ctx, ctx) // no need for 2 different contexts
