@@ -1,23 +1,3 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2016 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
-
 package local
 
 import (
@@ -38,10 +18,6 @@ import (
 
 // ExecutionScheduler is the local implementation of lib.ExecutionScheduler
 type ExecutionScheduler struct {
-	runner  lib.Runner
-	options lib.Options
-	logger  logrus.FieldLogger
-
 	initProgress    *pb.ProgressBar
 	executorConfigs []lib.ExecutorConfig // sorted by (startTime, ID)
 	executors       []lib.Executor       // sorted by (startTime, ID), excludes executors with no work
@@ -62,10 +38,8 @@ var _ lib.ExecutionScheduler = &ExecutionScheduler{}
 // instance, without initializing it beyond the bare minimum. Specifically, it
 // creates the needed executor instances and a lot of state placeholders, but it
 // doesn't initialize the executors and it doesn't initialize or run VUs.
-func NewExecutionScheduler(
-	runner lib.Runner, builtinMetrics *metrics.BuiltinMetrics, logger logrus.FieldLogger,
-) (*ExecutionScheduler, error) {
-	options := runner.GetOptions()
+func NewExecutionScheduler(trs *lib.TestRunState) (*ExecutionScheduler, error) {
+	options := trs.Options
 	et, err := lib.NewExecutionTuple(options.ExecutionSegment, options.ExecutionSegmentSequence)
 	if err != nil {
 		return nil, err
@@ -74,7 +48,7 @@ func NewExecutionScheduler(
 	maxPlannedVUs := lib.GetMaxPlannedVUs(executionPlan)
 	maxPossibleVUs := lib.GetMaxPossibleVUs(executionPlan)
 
-	executionState := lib.NewExecutionState(options, et, builtinMetrics, maxPlannedVUs, maxPossibleVUs)
+	executionState := lib.NewExecutionState(trs, et, maxPlannedVUs, maxPossibleVUs)
 	maxDuration, _ := lib.GetEndOffset(executionPlan) // we don't care if the end offset is final
 
 	executorConfigs := options.Scenarios.GetSortedConfigs()
@@ -82,13 +56,13 @@ func NewExecutionScheduler(
 	// Only take executors which have work.
 	for _, sc := range executorConfigs {
 		if !sc.HasWork(et) {
-			logger.Warnf(
+			trs.Logger.Warnf(
 				"Executor '%s' is disabled for segment %s due to lack of work!",
 				sc.GetName(), options.ExecutionSegment,
 			)
 			continue
 		}
-		s, err := sc.NewExecutor(executionState, logger.WithFields(logrus.Fields{
+		s, err := sc.NewExecutor(executionState, trs.Logger.WithFields(logrus.Fields{
 			"scenario": sc.GetName(),
 			"executor": sc.GetType(),
 		}))
@@ -105,10 +79,6 @@ func NewExecutionScheduler(
 	}
 
 	return &ExecutionScheduler{
-		runner:  runner,
-		logger:  logger,
-		options: options,
-
 		initProgress:    pb.New(pb.WithConstLeft("Init")),
 		executors:       executors,
 		executorConfigs: executorConfigs,
@@ -123,8 +93,8 @@ func NewExecutionScheduler(
 }
 
 // GetRunner returns the wrapped lib.Runner instance.
-func (e *ExecutionScheduler) GetRunner() lib.Runner {
-	return e.runner
+func (e *ExecutionScheduler) GetRunner() lib.Runner { // TODO: remove
+	return e.state.Test.Runner
 }
 
 // GetState returns a pointer to the execution state struct for the local
@@ -165,12 +135,12 @@ func (e *ExecutionScheduler) GetExecutionPlan() []lib.ExecutionStep {
 // in the Init() method, and also passed to executors so they can initialize
 // any unplanned VUs themselves.
 func (e *ExecutionScheduler) initVU(
-	samplesOut chan<- metrics.SampleContainer, logger *logrus.Entry,
+	samplesOut chan<- metrics.SampleContainer, logger logrus.FieldLogger,
 ) (lib.InitializedVU, error) {
 	// Get the VU IDs here, so that the VUs are (mostly) ordered by their
 	// number in the channel buffer
 	vuIDLocal, vuIDGlobal := e.state.GetUniqueVUIdentifiers()
-	vu, err := e.runner.NewVU(vuIDLocal, vuIDGlobal, samplesOut)
+	vu, err := e.state.Test.Runner.NewVU(vuIDLocal, vuIDGlobal, samplesOut)
 	if err != nil {
 		return nil, errext.WithHint(err, fmt.Sprintf("error while initializing VU #%d", vuIDGlobal))
 	}
@@ -201,7 +171,7 @@ func (e *ExecutionScheduler) getRunStats() string {
 
 func (e *ExecutionScheduler) initVUsConcurrently(
 	ctx context.Context, samplesOut chan<- metrics.SampleContainer, count uint64,
-	concurrency int, logger *logrus.Entry,
+	concurrency int, logger logrus.FieldLogger,
 ) chan error {
 	doneInits := make(chan error, count) // poor man's early-return waitgroup
 	limiter := make(chan struct{})
@@ -233,7 +203,8 @@ func (e *ExecutionScheduler) initVUsConcurrently(
 }
 
 func (e *ExecutionScheduler) emitVUsAndVUsMax(ctx context.Context, out chan<- metrics.SampleContainer) {
-	e.logger.Debug("Starting emission of VUs and VUsMax metrics...")
+	e.state.Test.Logger.Debug("Starting emission of VUs and VUsMax metrics...")
+	runTags := metrics.NewSampleTags(e.state.Test.Options.RunTags)
 
 	emitMetrics := func() {
 		t := time.Now()
@@ -241,17 +212,17 @@ func (e *ExecutionScheduler) emitVUsAndVUsMax(ctx context.Context, out chan<- me
 			Samples: []metrics.Sample{
 				{
 					Time:   t,
-					Metric: e.state.BuiltinMetrics.VUs,
+					Metric: e.state.Test.BuiltinMetrics.VUs,
 					Value:  float64(e.state.GetCurrentlyActiveVUsCount()),
-					Tags:   e.options.RunTags,
+					Tags:   runTags,
 				}, {
 					Time:   t,
-					Metric: e.state.BuiltinMetrics.VUsMax,
+					Metric: e.state.Test.BuiltinMetrics.VUsMax,
 					Value:  float64(e.state.GetInitializedVUsCount()),
-					Tags:   e.options.RunTags,
+					Tags:   runTags,
 				},
 			},
-			Tags: e.options.RunTags,
+			Tags: runTags,
 			Time: t,
 		}
 		metrics.PushIfNotDone(ctx, out, samples)
@@ -261,7 +232,7 @@ func (e *ExecutionScheduler) emitVUsAndVUsMax(ctx context.Context, out chan<- me
 	go func() {
 		defer func() {
 			ticker.Stop()
-			e.logger.Debug("Metrics emission of VUs and VUsMax metrics stopped")
+			e.state.Test.Logger.Debug("Metrics emission of VUs and VUsMax metrics stopped")
 			close(e.vusEmissionStopped)
 		}()
 
@@ -283,7 +254,7 @@ func (e *ExecutionScheduler) emitVUsAndVUsMax(ctx context.Context, out chan<- me
 func (e *ExecutionScheduler) Init(ctx context.Context, samplesOut chan<- metrics.SampleContainer) error {
 	e.emitVUsAndVUsMax(ctx, samplesOut)
 
-	logger := e.logger.WithField("phase", "local-execution-scheduler-init")
+	logger := e.state.Test.Logger.WithField("phase", "local-execution-scheduler-init")
 	vusToInitialize := lib.GetMaxPlannedVUs(e.executionPlan)
 	logger.WithFields(logrus.Fields{
 		"neededVUs":      vusToInitialize,
@@ -350,7 +321,7 @@ func (e *ExecutionScheduler) runExecutor(
 ) {
 	executorConfig := executor.GetConfig()
 	executorStartTime := executorConfig.GetStartTime()
-	executorLogger := e.logger.WithFields(logrus.Fields{
+	executorLogger := e.state.Test.Logger.WithFields(logrus.Fields{
 		"executor":  executorConfig.GetName(),
 		"type":      executorConfig.GetType(),
 		"startTime": executorStartTime,
@@ -402,7 +373,7 @@ func (e *ExecutionScheduler) Run(globalCtx, runCtx context.Context, engineOut ch
 	}()
 
 	executorsCount := len(e.executors)
-	logger := e.logger.WithField("phase", "local-execution-scheduler-run")
+	logger := e.state.Test.Logger.WithField("phase", "local-execution-scheduler-run")
 	e.initProgress.Modify(pb.WithConstLeft("Run"))
 	var interrupted bool
 	defer func() {
@@ -436,11 +407,11 @@ func (e *ExecutionScheduler) Run(globalCtx, runCtx context.Context, engineOut ch
 	defer cancel() // just in case, and to shut up go vet...
 
 	// Run setup() before any executors, if it's not disabled
-	if !e.options.NoSetup.Bool {
+	if !e.state.Test.Options.NoSetup.Bool {
 		logger.Debug("Running setup()")
 		e.state.SetExecutionStatus(lib.ExecutionStatusSetup)
 		e.initProgress.Modify(pb.WithConstProgress(1, "setup()"))
-		if err := e.runner.Setup(runSubCtx, engineOut); err != nil {
+		if err := e.state.Test.Runner.Setup(runSubCtx, engineOut); err != nil {
 			logger.WithField("error", err).Debug("setup() aborted by error")
 			return err
 		}
@@ -472,14 +443,14 @@ func (e *ExecutionScheduler) Run(globalCtx, runCtx context.Context, engineOut ch
 	}
 
 	// Run teardown() after all executors are done, if it's not disabled
-	if !e.options.NoTeardown.Bool {
+	if !e.state.Test.Options.NoTeardown.Bool {
 		logger.Debug("Running teardown()")
 		e.state.SetExecutionStatus(lib.ExecutionStatusTeardown)
 		e.initProgress.Modify(pb.WithConstProgress(1, "teardown()"))
 
 		// We run teardown() with the global context, so it isn't interrupted by
 		// aborts caused by thresholds or even Ctrl+C (unless used twice).
-		if err := e.runner.Teardown(globalCtx, engineOut); err != nil {
+		if err := e.state.Test.Runner.Teardown(globalCtx, engineOut); err != nil {
 			logger.WithField("error", err).Debug("teardown() aborted by error")
 			return err
 		}
@@ -499,7 +470,7 @@ func (e *ExecutionScheduler) SetPaused(pause bool) error {
 		if pause {
 			return fmt.Errorf("execution is already paused")
 		}
-		e.logger.Debug("Starting execution")
+		e.state.Test.Logger.Debug("Starting execution")
 		return e.state.Resume()
 	}
 
