@@ -1,23 +1,3 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2016 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
-
 package js
 
 import (
@@ -63,54 +43,53 @@ type Bundle struct {
 // A BundleInstance is a self-contained instance of a Bundle.
 type BundleInstance struct {
 	Runtime *goja.Runtime
-	Context *context.Context
 
 	// TODO: maybe just have a reference to the Bundle? or save and pass rtOpts?
 	env map[string]string
 
-	exports map[string]goja.Callable
+	exports      map[string]goja.Callable
+	moduleVUImpl *moduleVUImpl
+	pgm          programWithSource
 }
 
 // NewBundle creates a new bundle from a source file and a filesystem.
 func NewBundle(
-	logger logrus.FieldLogger, src *loader.SourceData, filesystems map[string]afero.Fs, rtOpts lib.RuntimeOptions,
-	registry *metrics.Registry,
+	piState *lib.TestPreInitState, src *loader.SourceData, filesystems map[string]afero.Fs,
 ) (*Bundle, error) {
-	compatMode, err := lib.ValidateCompatibilityMode(rtOpts.CompatibilityMode.String)
+	compatMode, err := lib.ValidateCompatibilityMode(piState.RuntimeOptions.CompatibilityMode.String)
 	if err != nil {
 		return nil, err
 	}
 
 	// Compile sources, both ES5 and ES6 are supported.
 	code := string(src.Data)
-	c := compiler.New(logger)
+	c := compiler.New(piState.Logger)
 	c.Options = compiler.Options{
 		CompatibilityMode: compatMode,
 		Strict:            true,
-		SourceMapLoader:   generateSourceMapLoader(logger, filesystems),
+		SourceMapLoader:   generateSourceMapLoader(piState.Logger, filesystems),
 	}
-	pgm, _, err := c.Compile(code, src.URL.String(), true)
+	pgm, _, err := c.Compile(code, src.URL.String(), false)
 	if err != nil {
 		return nil, err
 	}
 	// Make a bundle, instantiate it into a throwaway VM to populate caches.
 	rt := goja.New()
 	bundle := Bundle{
-		Filename: src.URL,
-		Source:   code,
-		Program:  pgm,
-		BaseInitContext: NewInitContext(logger, rt, c, compatMode, new(context.Context),
-			filesystems, loader.Dir(src.URL)),
-		RuntimeOptions:    rtOpts,
+		Filename:          src.URL,
+		Source:            code,
+		Program:           pgm,
+		BaseInitContext:   NewInitContext(piState.Logger, rt, c, compatMode, filesystems, loader.Dir(src.URL)),
+		RuntimeOptions:    piState.RuntimeOptions,
 		CompatibilityMode: compatMode,
 		exports:           make(map[string]goja.Callable),
-		registry:          registry,
+		registry:          piState.Registry,
 	}
-	if err = bundle.instantiate(logger, rt, bundle.BaseInitContext, 0); err != nil {
+	if err = bundle.instantiate(piState.Logger, rt, bundle.BaseInitContext, 0); err != nil {
 		return nil, err
 	}
 
-	err = bundle.getExports(logger, rt, true)
+	err = bundle.getExports(piState.Logger, rt, true)
 	if err != nil {
 		return nil, err
 	}
@@ -119,13 +98,12 @@ func NewBundle(
 }
 
 // NewBundleFromArchive creates a new bundle from an lib.Archive.
-func NewBundleFromArchive(
-	logger logrus.FieldLogger, arc *lib.Archive, rtOpts lib.RuntimeOptions, registry *metrics.Registry,
-) (*Bundle, error) {
+func NewBundleFromArchive(piState *lib.TestPreInitState, arc *lib.Archive) (*Bundle, error) {
 	if arc.Type != "js" {
 		return nil, fmt.Errorf("expected bundle type 'js', got '%s'", arc.Type)
 	}
 
+	rtOpts := piState.RuntimeOptions // copy the struct from the TestPreInitState
 	if !rtOpts.CompatibilityMode.Valid {
 		// `k6 run --compatibility-mode=whatever archive.tar` should override
 		// whatever value is in the archive
@@ -136,19 +114,18 @@ func NewBundleFromArchive(
 		return nil, err
 	}
 
-	c := compiler.New(logger)
+	c := compiler.New(piState.Logger)
 	c.Options = compiler.Options{
 		Strict:            true,
 		CompatibilityMode: compatMode,
-		SourceMapLoader:   generateSourceMapLoader(logger, arc.Filesystems),
+		SourceMapLoader:   generateSourceMapLoader(piState.Logger, arc.Filesystems),
 	}
-	pgm, _, err := c.Compile(string(arc.Data), arc.FilenameURL.String(), true)
+	pgm, _, err := c.Compile(string(arc.Data), arc.FilenameURL.String(), false)
 	if err != nil {
 		return nil, err
 	}
 	rt := goja.New()
-	initctx := NewInitContext(logger, rt, c, compatMode,
-		new(context.Context), arc.Filesystems, arc.PwdURL)
+	initctx := NewInitContext(piState.Logger, rt, c, compatMode, arc.Filesystems, arc.PwdURL)
 
 	env := arc.Env
 	if env == nil {
@@ -169,16 +146,16 @@ func NewBundleFromArchive(
 		RuntimeOptions:    rtOpts,
 		CompatibilityMode: compatMode,
 		exports:           make(map[string]goja.Callable),
-		registry:          registry,
+		registry:          piState.Registry,
 	}
 
-	if err = bundle.instantiate(logger, rt, bundle.BaseInitContext, 0); err != nil {
+	if err = bundle.instantiate(piState.Logger, rt, bundle.BaseInitContext, 0); err != nil {
 		return nil, err
 	}
 
 	// Grab exported objects, but avoid overwriting options, which would
 	// be initialized from the metadata.json at this point.
-	err = bundle.getExports(logger, rt, false)
+	err = bundle.getExports(piState.Logger, rt, false)
 	if err != nil {
 		return nil, err
 	}
@@ -209,7 +186,8 @@ func (b *Bundle) makeArchive() *lib.Archive {
 
 // getExports validates and extracts exported objects
 func (b *Bundle) getExports(logger logrus.FieldLogger, rt *goja.Runtime, options bool) error {
-	exportsV := rt.Get("exports")
+	pgm := b.BaseInitContext.programs[b.Filename.String()] // this is the main script and it's always present
+	exportsV := pgm.module.Get("exports")
 	if goja.IsNull(exportsV) || goja.IsUndefined(exportsV) {
 		return errors.New("exports must be an object")
 	}
@@ -253,41 +231,46 @@ func (b *Bundle) getExports(logger logrus.FieldLogger, rt *goja.Runtime, options
 }
 
 // Instantiate creates a new runtime from this bundle.
-func (b *Bundle) Instantiate(
-	logger logrus.FieldLogger, vuID uint64, vuImpl *moduleVUImpl,
-) (bi *BundleInstance, instErr error) {
+func (b *Bundle) Instantiate(logger logrus.FieldLogger, vuID uint64) (*BundleInstance, error) {
 	// Instantiate the bundle into a new VM using a bound init context. This uses a context with a
 	// runtime, but no state, to allow module-provided types to function within the init context.
-	vuImpl.runtime = goja.New()
+	vuImpl := &moduleVUImpl{runtime: goja.New()}
 	init := newBoundInitContext(b.BaseInitContext, vuImpl)
 	if err := b.instantiate(logger, vuImpl.runtime, init, vuID); err != nil {
 		return nil, err
 	}
 
 	rt := vuImpl.runtime
-	bi = &BundleInstance{
-		Runtime: rt,
-		Context: &vuImpl.ctx,
-		exports: make(map[string]goja.Callable),
-		env:     b.RuntimeOptions.Env,
+	pgm := init.programs[b.Filename.String()] // this is the main script and it's always present
+	bi := &BundleInstance{
+		Runtime:      rt,
+		exports:      make(map[string]goja.Callable),
+		env:          b.RuntimeOptions.Env,
+		moduleVUImpl: vuImpl,
+		pgm:          pgm,
 	}
 
 	// Grab any exported functions that could be executed. These were
 	// already pre-validated in cmd.validateScenarioConfig(), just get them here.
-	exports := rt.Get("exports").ToObject(rt)
+	exports := pgm.module.Get("exports").ToObject(rt)
 	for k := range b.exports {
 		fn, _ := goja.AssertFunction(exports.Get(k))
 		bi.exports[k] = fn
 	}
 
-	jsOptions := rt.Get("options")
+	jsOptions := exports.Get("options")
 	var jsOptionsObj *goja.Object
 	if jsOptions == nil || goja.IsNull(jsOptions) || goja.IsUndefined(jsOptions) {
 		jsOptionsObj = rt.NewObject()
-		rt.Set("options", jsOptionsObj)
+		err := exports.Set("options", jsOptionsObj)
+		if err != nil {
+			return nil, fmt.Errorf("couldn't set exported options with merged values: %w", err)
+		}
 	} else {
 		jsOptionsObj = jsOptions.ToObject(rt)
 	}
+
+	var instErr error
 	b.Options.ForEachSpecified("json", func(key string, val interface{}) {
 		if err := jsOptionsObj.Set(key, val); err != nil {
 			instErr = err
@@ -299,16 +282,23 @@ func (b *Bundle) Instantiate(
 
 // Instantiates the bundle into an existing runtime. Not public because it also messes with a bunch
 // of other things, will potentially thrash data and makes a mess in it if the operation fails.
+
+func (b *Bundle) initializeProgramObject(rt *goja.Runtime, init *InitContext) programWithSource {
+	pgm := programWithSource{
+		pgm:     b.Program,
+		src:     b.Source,
+		exports: rt.NewObject(),
+		module:  rt.NewObject(),
+	}
+	_ = pgm.module.Set("exports", pgm.exports)
+	init.programs[b.Filename.String()] = pgm
+	return pgm
+}
+
 func (b *Bundle) instantiate(logger logrus.FieldLogger, rt *goja.Runtime, init *InitContext, vuID uint64) (err error) {
 	rt.SetFieldNameMapper(common.FieldNameMapper{})
 	rt.SetRandSource(common.NewRandSource())
 
-	exports := rt.NewObject()
-	rt.Set("exports", exports)
-	module := rt.NewObject()
-	_ = module.Set("exports", exports)
-	rt.Set("module", module)
-
 	env := make(map[string]string, len(b.RuntimeOptions.Env))
 	for key, value := range b.RuntimeOptions.Env {
 		env[key] = value
@@ -321,22 +311,31 @@ func (b *Bundle) instantiate(logger logrus.FieldLogger, rt *goja.Runtime, init *
 		rt.Set("global", rt.GlobalObject())
 	}
 
-	// TODO: get rid of the unused ctxPtr, use a real external context (so we
-	// can interrupt), build the common.InitEnvironment earlier and reuse it
 	initenv := &common.InitEnvironment{
 		Logger:      logger,
 		FileSystems: init.filesystems,
 		CWD:         init.pwd,
 		Registry:    b.registry,
 	}
-	init.moduleVUImpl.initEnv = initenv
-	init.moduleVUImpl.ctx = context.Background()
 	unbindInit := b.setInitGlobals(rt, init)
+	init.moduleVUImpl.ctx = context.Background()
+	init.moduleVUImpl.initEnv = initenv
 	init.moduleVUImpl.eventLoop = eventloop.New(init.moduleVUImpl)
+	pgm := b.initializeProgramObject(rt, init)
+
 	err = common.RunWithPanicCatching(logger, rt, func() error {
 		return init.moduleVUImpl.eventLoop.Start(func() error {
-			_, errRun := rt.RunProgram(b.Program)
-			return errRun
+			f, errRun := rt.RunProgram(b.Program)
+			if errRun != nil {
+				return errRun
+			}
+			if call, ok := goja.AssertFunction(f); ok {
+				if _, errRun = call(pgm.exports, pgm.module, pgm.exports); errRun != nil {
+					return errRun
+				}
+				return nil
+			}
+			panic("Somehow a commonjs main module is not wrapped in a function")
 		})
 	})
 
@@ -347,6 +346,12 @@ func (b *Bundle) instantiate(logger logrus.FieldLogger, rt *goja.Runtime, init *
 		}
 		return err
 	}
+	exportsV := pgm.module.Get("exports")
+	if goja.IsNull(exportsV) {
+		return errors.New("exports must be an object")
+	}
+	pgm.exports = exportsV.ToObject(rt)
+	init.programs[b.Filename.String()] = pgm
 	unbindInit()
 	init.moduleVUImpl.ctx = nil
 	init.moduleVUImpl.initEnv = nil
