@@ -1,23 +1,3 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2020 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
-
 package modules
 
 import (
@@ -95,20 +75,14 @@ type VU interface {
 	// Runtime returns the goja.Runtime for the current VU
 	Runtime() *goja.Runtime
 
-	// RegisterCallback lets a module declare that it wants to run a function on the event loop *at a later point in time*.
-	// It needs to be called from within the event loop, so not in a goroutine spawned by a module.
-	// Its result can be called with a function that will be executed *on the event loop* -
-	// possibly letting you call RegisterCallback again.
-	// Calling the result can be done at any time. The event loop will block until that happens, (if it doesn't
-	// have something else to run) so in the event of an iteration end or abort (for example due to an exception),
-	// It is the module responsibility to monitor the context and abort on it being done.
-	// This still means that the returned function here *needs* to be called to signal that the module
-	// has aborted the operation and will not do anything more, not doing so will block k6.
+	// RegisterCallback lets a JS module declare that it wants to run a function
+	// on the event loop *at a later point in time*. See the documentation for
+	// `EventLoop.RegisterCallback()` in the `k6/js/eventloop` Go module for
+	// the very important details on its usage and restrictions.
 	//
-	// Experimental
-	//
-	// Notice: This API is EXPERIMENTAL and may be changed or removed in a later release.
-	RegisterCallback() func(func() error)
+	// Notice: This API is EXPERIMENTAL and may be changed, renamed or
+	// completely removed in a later k6 release.
+	RegisterCallback() (enqueueCallback func(func() error))
 
 	// sealing field will help probably with pointing users that they just need to embed this in their Instance
 	// implementations
