@@ -1,36 +1,22 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2016 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
-
 package js
 
 import (
 	"bytes"
 	"context"
+	"crypto/rand"
+	"crypto/rsa"
 	"crypto/tls"
 	"crypto/x509"
+	"crypto/x509/pkix"
+	"encoding/pem"
 	"fmt"
 	"go/build"
 	"io/ioutil"
 	stdlog "log"
+	"math/big"
 	"net"
 	"net/http"
+	"net/http/httptest"
 	"net/url"
 	"os"
 	"strings"
@@ -43,6 +29,7 @@ import (
 	"github.com/spf13/afero"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"golang.org/x/time/rate"
 	"google.golang.org/grpc/test/grpc_testing"
 	"gopkg.in/guregu/null.v3"
 
@@ -70,8 +57,8 @@ func TestRunnerNew(t *testing.T) {
 	t.Run("Valid", func(t *testing.T) {
 		t.Parallel()
 		r, err := getSimpleRunner(t, "/script.js", `
-			var counter = 0;
-			exports.default = function() { counter++; }
+			exports.counter = 0;
+			exports.default = function() { exports.counter++; }
 		`)
 		require.NoError(t, err)
 
@@ -81,7 +68,7 @@ func TestRunnerNew(t *testing.T) {
 			require.NoError(t, err)
 			vuc, ok := initVU.(*VU)
 			require.True(t, ok)
-			assert.Equal(t, int64(0), vuc.Runtime.Get("counter").Export())
+			assert.Equal(t, int64(0), vuc.pgm.exports.Get("counter").Export())
 
 			ctx, cancel := context.WithCancel(context.Background())
 			defer cancel()
@@ -89,7 +76,7 @@ func TestRunnerNew(t *testing.T) {
 			t.Run("RunOnce", func(t *testing.T) {
 				err = vu.RunOnce()
 				require.NoError(t, err)
-				assert.Equal(t, int64(1), vuc.Runtime.Get("counter").Export())
+				assert.Equal(t, int64(1), vuc.pgm.exports.Get("counter").Export())
 			})
 		})
 	})
@@ -97,7 +84,7 @@ func TestRunnerNew(t *testing.T) {
 	t.Run("Invalid", func(t *testing.T) {
 		t.Parallel()
 		_, err := getSimpleRunner(t, "/script.js", `blarg`)
-		assert.EqualError(t, err, "ReferenceError: blarg is not defined\n\tat file:///script.js:1:1(0)\n")
+		assert.EqualError(t, err, "ReferenceError: blarg is not defined\n\tat file:///script.js:2:1(1)\n\tat native\n")
 	})
 }
 
@@ -110,7 +97,7 @@ func TestRunnerGetDefaultGroup(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -127,7 +114,7 @@ func TestRunnerOptions(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -151,15 +138,59 @@ func TestRunnerOptions(t *testing.T) {
 	}
 }
 
+func TestRunnerRPSLimit(t *testing.T) {
+	t.Parallel()
+
+	var nilLimiter *rate.Limiter
+
+	variants := []struct {
+		name    string
+		options lib.Options
+		limiter *rate.Limiter
+	}{
+		{
+			name:    "RPS not defined",
+			options: lib.Options{},
+			limiter: nilLimiter,
+		},
+		{
+			name:    "RPS set to non-zero int",
+			options: lib.Options{RPS: null.IntFrom(9)},
+			limiter: rate.NewLimiter(rate.Limit(9), 1),
+		},
+		{
+			name:    "RPS set to zero",
+			options: lib.Options{RPS: null.IntFrom(0)},
+			limiter: nilLimiter,
+		},
+		{
+			name:    "RPS set to below zero value",
+			options: lib.Options{RPS: null.IntFrom(-1)},
+			limiter: nilLimiter,
+		},
+	}
+
+	for _, variant := range variants {
+		variant := variant
+
+		t.Run(variant.name, func(t *testing.T) {
+			t.Parallel()
+
+			r, err := getSimpleRunner(t, "/script.js", `exports.default = function() {};`)
+			require.NoError(t, err)
+			err = r.SetOptions(variant.options)
+			require.NoError(t, err)
+			assert.Equal(t, variant.limiter, r.RPSLimit)
+		})
+	}
+}
+
 func TestOptionsSettingToScript(t *testing.T) {
 	t.Parallel()
 
 	optionVariants := []string{
-		"",
-		"var options = null;",
-		"var options = undefined;",
-		"var options = {};",
-		"var options = {teardownTimeout: '1s'};",
+		"export var options = {};",
+		"export var options = {teardownTimeout: '1s'};",
 	}
 
 	for i, variant := range optionVariants {
@@ -220,7 +251,7 @@ func TestOptionsPropagationToScript(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -313,15 +344,17 @@ func TestSetupDataIsolation(t *testing.T) {
 	options := runner.GetOptions()
 	require.Empty(t, options.Validate())
 
-	registry := metrics.NewRegistry()
-	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
-	execScheduler, err := local.NewExecutionScheduler(runner, builtinMetrics, testutils.NewLogger(t))
+	testRunState := &lib.TestRunState{
+		TestPreInitState: runner.preInitState,
+		Options:          options,
+		Runner:           runner,
+	}
+
+	execScheduler, err := local.NewExecutionScheduler(testRunState)
 	require.NoError(t, err)
 
 	mockOutput := mockoutput.New()
-	engine, err := core.NewEngine(
-		execScheduler, options, lib.RuntimeOptions{}, []output.Output{mockOutput}, testutils.NewLogger(t), registry,
-	)
+	engine, err := core.NewEngine(testRunState, execScheduler, []output.Output{mockOutput})
 	require.NoError(t, err)
 	require.NoError(t, engine.OutputManager.StartOutputs())
 	defer engine.OutputManager.StopOutputs()
@@ -514,7 +547,7 @@ func TestRunnerIntegrationImports(t *testing.T) {
 				registry := metrics.NewRegistry()
 				builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 				r2, err := NewFromArchive(
-					&lib.RuntimeState{
+					&lib.TestPreInitState{
 						Logger:         testutils.NewLogger(t),
 						BuiltinMetrics: builtinMetrics,
 						Registry:       registry,
@@ -551,7 +584,7 @@ func TestVURunContext(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -602,7 +635,7 @@ func TestVURunInterrupt(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -644,7 +677,7 @@ func TestVURunInterruptDoesntPanic(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -667,7 +700,7 @@ func TestVURunInterruptDoesntPanic(t *testing.T) {
 
 			initVU, err := r.newVU(1, 1, samples)
 			require.NoError(t, err)
-			for i := 0; i < 1000; i++ {
+			for i := 0; i < 100; i++ {
 				wg.Add(1)
 				newCtx, newCancel := context.WithCancel(ctx)
 				vu := initVU.Activate(&lib.VUActivationParams{
@@ -709,7 +742,7 @@ func TestVUIntegrationGroups(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -769,7 +802,7 @@ func TestVUIntegrationMetrics(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -819,8 +852,127 @@ func TestVUIntegrationMetrics(t *testing.T) {
 	}
 }
 
+func GenerateTLSCertificate(t *testing.T, host string, notBefore time.Time, validFor time.Duration) ([]byte, []byte) {
+	priv, err := rsa.GenerateKey(rand.Reader, 2048)
+	require.NoError(t, err)
+
+	// ECDSA, ED25519 and RSA subject keys should have the DigitalSignature
+	// KeyUsage bits set in the x509.Certificate template
+	keyUsage := x509.KeyUsageDigitalSignature
+	// Only RSA subject keys should have the KeyEncipherment KeyUsage bits set. In
+	// the context of TLS this KeyUsage is particular to RSA key exchange and
+	// authentication.
+	keyUsage |= x509.KeyUsageKeyEncipherment
+
+	notAfter := notBefore.Add(validFor)
+
+	serialNumberLimit := new(big.Int).Lsh(big.NewInt(1), 128)
+	serialNumber, err := rand.Int(rand.Reader, serialNumberLimit)
+	require.NoError(t, err)
+
+	template := x509.Certificate{
+		SerialNumber: serialNumber,
+		Subject: pkix.Name{
+			Organization: []string{"Acme Co"},
+		},
+		NotBefore: notBefore,
+		NotAfter:  notAfter,
+
+		KeyUsage:              keyUsage,
+		ExtKeyUsage:           []x509.ExtKeyUsage{x509.ExtKeyUsageServerAuth},
+		BasicConstraintsValid: true,
+		SignatureAlgorithm:    x509.SHA256WithRSA,
+	}
+
+	hosts := strings.Split(host, ",")
+	for _, h := range hosts {
+		if ip := net.ParseIP(h); ip != nil {
+			template.IPAddresses = append(template.IPAddresses, ip)
+		} else {
+			template.DNSNames = append(template.DNSNames, h)
+		}
+	}
+
+	template.IsCA = true
+	template.KeyUsage |= x509.KeyUsageCertSign
+
+	derBytes, err := x509.CreateCertificate(rand.Reader, &template, &template, &priv.PublicKey, priv)
+	require.NoError(t, err)
+
+	certPem := pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: derBytes})
+	require.NoError(t, err)
+
+	privBytes, err := x509.MarshalPKCS8PrivateKey(priv)
+	require.NoError(t, err)
+	keyPem := pem.EncodeToMemory(&pem.Block{Type: "PRIVATE KEY", Bytes: privBytes})
+	require.NoError(t, err)
+	return certPem, keyPem
+}
+
+func GetTestServerWithCertificate(t *testing.T, certPem, key []byte) *httptest.Server {
+	server := &http.Server{
+		Handler: http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			w.WriteHeader(200)
+		}),
+		ReadHeaderTimeout: time.Second,
+		ReadTimeout:       time.Second,
+	}
+	s := &httptest.Server{}
+	s.Config = server
+
+	s.TLS = new(tls.Config)
+	if s.TLS.NextProtos == nil {
+		nextProtos := []string{"http/1.1"}
+		if s.EnableHTTP2 {
+			nextProtos = []string{"h2"}
+		}
+		s.TLS.NextProtos = nextProtos
+	}
+	cert, err := tls.X509KeyPair(certPem, key)
+	require.NoError(t, err)
+	s.TLS.Certificates = append(s.TLS.Certificates, cert)
+	for _, suite := range tls.CipherSuites() {
+		if !strings.Contains(suite.Name, "256") {
+			continue
+		}
+		s.TLS.CipherSuites = append(s.TLS.CipherSuites, suite.ID)
+	}
+	certpool := x509.NewCertPool()
+	certificate, err := x509.ParseCertificate(cert.Certificate[0])
+	require.NoError(t, err)
+	certpool.AddCert(certificate)
+	client := &http.Client{Transport: &http.Transport{}}
+	client.Transport = &http.Transport{
+		TLSClientConfig: &tls.Config{ //nolint:gosec
+			RootCAs: certpool,
+		},
+		ForceAttemptHTTP2: s.EnableHTTP2,
+	}
+	s.Listener, err = net.Listen("tcp", "")
+	require.NoError(t, err)
+	s.Listener = tls.NewListener(s.Listener, s.TLS)
+	s.URL = "https://" + s.Listener.Addr().String()
+	return s
+}
+
 func TestVUIntegrationInsecureRequests(t *testing.T) {
 	t.Parallel()
+	certPem, keyPem := GenerateTLSCertificate(t, "mybadssl.localhost", time.Now(), 0)
+	s := GetTestServerWithCertificate(t, certPem, keyPem)
+	go func() {
+		_ = s.Config.Serve(s.Listener)
+	}()
+	t.Cleanup(func() {
+		require.NoError(t, s.Config.Close())
+	})
+	host, port, err := net.SplitHostPort(s.Listener.Addr().String())
+	require.NoError(t, err)
+	ip := net.ParseIP(host)
+	mybadsslHostname, err := lib.NewHostAddress(ip, port)
+	require.NoError(t, err)
+	cert, err := x509.ParseCertificate(s.TLS.Certificates[0].Certificate[0])
+	require.NoError(t, err)
+
 	testdata := map[string]struct {
 		opts   lib.Options
 		errMsg string
@@ -843,16 +995,19 @@ func TestVUIntegrationInsecureRequests(t *testing.T) {
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 			r1, err := getSimpleRunner(t, "/script.js", `
-					var http = require("k6/http");;
-					exports.default = function() { http.get("https://expired.badssl.com/"); }
+			  var http = require("k6/http");;
+        exports.default = function() { http.get("https://mybadssl.localhost/"); }
 				`)
 			require.NoError(t, err)
 			require.NoError(t, r1.SetOptions(lib.Options{Throw: null.BoolFrom(true)}.Apply(data.opts)))
 
+			r1.Bundle.Options.Hosts = map[string]*lib.HostAddress{
+				"mybadssl.localhost": mybadsslHostname,
+			}
 			registry := metrics.NewRegistry()
 			builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 			r2, err := NewFromArchive(
-				&lib.RuntimeState{
+				&lib.TestPreInitState{
 					Logger:         testutils.NewLogger(t),
 					BuiltinMetrics: builtinMetrics,
 					Registry:       registry,
@@ -863,10 +1018,12 @@ func TestVUIntegrationInsecureRequests(t *testing.T) {
 				r := r
 				t.Run(name, func(t *testing.T) {
 					t.Parallel()
-					r.Logger, _ = logtest.NewNullLogger()
+					r.preInitState.Logger, _ = logtest.NewNullLogger()
 
 					initVU, err := r.NewVU(1, 1, make(chan metrics.SampleContainer, 100))
 					require.NoError(t, err)
+					initVU.(*VU).TLSConfig.RootCAs = x509.NewCertPool() //nolint:forcetypeassert
+					initVU.(*VU).TLSConfig.RootCAs.AddCert(cert)        //nolint:forcetypeassert
 
 					ctx, cancel := context.WithCancel(context.Background())
 					defer cancel()
@@ -903,7 +1060,7 @@ func TestVUIntegrationBlacklistOption(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -944,7 +1101,7 @@ func TestVUIntegrationBlacklistScript(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -987,7 +1144,7 @@ func TestVUIntegrationBlockHostnamesOption(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -1030,7 +1187,7 @@ func TestVUIntegrationBlockHostnamesScript(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -1084,7 +1241,7 @@ func TestVUIntegrationHosts(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -1110,6 +1267,19 @@ func TestVUIntegrationHosts(t *testing.T) {
 
 func TestVUIntegrationTLSConfig(t *testing.T) {
 	t.Parallel()
+	certPem, keyPem := GenerateTLSCertificate(t, "sha256-badssl.localhost", time.Now(), time.Hour)
+	s := GetTestServerWithCertificate(t, certPem, keyPem)
+	go func() {
+		_ = s.Config.Serve(s.Listener)
+	}()
+	t.Cleanup(func() {
+		require.NoError(t, s.Config.Close())
+	})
+	host, port, err := net.SplitHostPort(s.Listener.Addr().String())
+	require.NoError(t, err)
+	ip := net.ParseIP(host)
+	mybadsslHostname, err := lib.NewHostAddress(ip, port)
+	require.NoError(t, err)
 	unsupportedVersionErrorMsg := "remote error: tls: handshake failure"
 	for _, tag := range build.Default.ReleaseTags {
 		if tag == "go1.12" {
@@ -1130,7 +1300,10 @@ func TestVUIntegrationTLSConfig(t *testing.T) {
 			"",
 		},
 		"UnsupportedCipherSuite": {
-			lib.Options{TLSCipherSuites: &lib.TLSCipherSuites{tls.TLS_RSA_WITH_RC4_128_SHA}},
+			lib.Options{
+				TLSCipherSuites: &lib.TLSCipherSuites{tls.TLS_RSA_WITH_RC4_128_SHA},
+				TLSVersion:      &lib.TLSVersions{Max: tls.VersionTLS12},
+			},
 			"remote error: tls: handshake failure",
 		},
 		"NullVersion": {
@@ -1148,19 +1321,24 @@ func TestVUIntegrationTLSConfig(t *testing.T) {
 	}
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
+	cert, err := x509.ParseCertificate(s.TLS.Certificates[0].Certificate[0])
+	require.NoError(t, err)
 	for name, data := range testdata {
 		data := data
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 			r1, err := getSimpleRunner(t, "/script.js", `
 					var http = require("k6/http");;
-					exports.default = function() { http.get("https://sha256.badssl.com/"); }
+					exports.default = function() { http.get("https://sha256-badssl.localhost/"); }
 				`)
 			require.NoError(t, err)
 			require.NoError(t, r1.SetOptions(lib.Options{Throw: null.BoolFrom(true)}.Apply(data.opts)))
 
+			r1.Bundle.Options.Hosts = map[string]*lib.HostAddress{
+				"sha256-badssl.localhost": mybadsslHostname,
+			}
 			r2, err := NewFromArchive(
-				&lib.RuntimeState{
+				&lib.TestPreInitState{
 					Logger:         testutils.NewLogger(t),
 					BuiltinMetrics: builtinMetrics,
 					Registry:       registry,
@@ -1172,16 +1350,18 @@ func TestVUIntegrationTLSConfig(t *testing.T) {
 				r := r
 				t.Run(name, func(t *testing.T) {
 					t.Parallel()
-					r.Logger, _ = logtest.NewNullLogger()
+					r.preInitState.Logger, _ = logtest.NewNullLogger()
 
 					initVU, err := r.NewVU(1, 1, make(chan metrics.SampleContainer, 100))
 					require.NoError(t, err)
+					initVU.(*VU).TLSConfig.RootCAs = x509.NewCertPool() //nolint:forcetypeassert
+					initVU.(*VU).TLSConfig.RootCAs.AddCert(cert)        //nolint:forcetypeassert
 					ctx, cancel := context.WithCancel(context.Background())
 					defer cancel()
 					vu := initVU.Activate(&lib.VUActivationParams{RunContext: ctx})
 					err = vu.RunOnce()
 					if data.errMsg != "" {
-						require.Error(t, err)
+						require.Error(t, err, "for message %q", data.errMsg)
 						assert.Contains(t, err.Error(), data.errMsg)
 					} else {
 						require.NoError(t, err)
@@ -1331,7 +1511,7 @@ func TestVUIntegrationCookiesReset(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -1391,7 +1571,7 @@ func TestVUIntegrationCookiesNoReset(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -1431,7 +1611,7 @@ func TestVUIntegrationVUID(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -1587,7 +1767,7 @@ func TestVUIntegrationClientCerts(t *testing.T) {
 			}
 			require.NoError(t, r1.SetOptions(opt))
 			r2, err := NewFromArchive(
-				&lib.RuntimeState{
+				&lib.TestPreInitState{
 					Logger:         testutils.NewLogger(t),
 					BuiltinMetrics: builtinMetrics,
 					Registry:       registry,
@@ -1599,7 +1779,7 @@ func TestVUIntegrationClientCerts(t *testing.T) {
 				r := r
 				t.Run(name, func(t *testing.T) {
 					t.Parallel()
-					r.Logger, _ = logtest.NewNullLogger()
+					r.preInitState.Logger, _ = logtest.NewNullLogger()
 					initVU, err := r.NewVU(1, 1, make(chan metrics.SampleContainer, 100))
 					require.NoError(t, err)
 					ctx, cancel := context.WithCancel(context.Background())
@@ -1752,7 +1932,7 @@ func TestArchiveRunningIntegrity(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -1797,7 +1977,7 @@ func TestArchiveNotPanicking(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -2010,7 +2190,7 @@ func TestVUPanic(t *testing.T) {
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	r2, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         testutils.NewLogger(t),
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -2078,7 +2258,7 @@ func runMultiFileTestCase(t *testing.T, tc multiFileTestCase, tb *httpmultibin.H
 	registry := metrics.NewRegistry()
 	builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 	runner, err := New(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         logger,
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -2120,7 +2300,7 @@ func runMultiFileTestCase(t *testing.T, tc multiFileTestCase, tb *httpmultibin.H
 
 	arc := runner.MakeArchive()
 	runnerFromArc, err := NewFromArchive(
-		&lib.RuntimeState{
+		&lib.TestPreInitState{
 			Logger:         logger,
 			BuiltinMetrics: builtinMetrics,
 			Registry:       registry,
@@ -2283,7 +2463,7 @@ func TestMinIterationDurationIsCancellable(t *testing.T) {
 	}
 }
 
-// nolint:paralleltest
+//nolint:paralleltest
 func TestForceHTTP1Feature(t *testing.T) {
 	cases := map[string]struct {
 		godebug               string
@@ -2345,7 +2525,7 @@ func TestForceHTTP1Feature(t *testing.T) {
 			registry := metrics.NewRegistry()
 			builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
 			r2, err := NewFromArchive(
-				&lib.RuntimeState{
+				&lib.TestPreInitState{
 					Logger:         testutils.NewLogger(t),
 					BuiltinMetrics: builtinMetrics,
 					Registry:       registry,
@@ -2441,9 +2621,13 @@ func TestExecutionInfo(t *testing.T) {
 			initVU, err := r.NewVU(1, 10, samples)
 			require.NoError(t, err)
 
-			registry := metrics.NewRegistry()
-			builtinMetrics := metrics.RegisterBuiltinMetrics(registry)
-			execScheduler, err := local.NewExecutionScheduler(r, builtinMetrics, testutils.NewLogger(t))
+			testRunState := &lib.TestRunState{
+				TestPreInitState: r.preInitState,
+				Options:          r.GetOptions(),
+				Runner:           r,
+			}
+
+			execScheduler, err := local.NewExecutionScheduler(testRunState)
 			require.NoError(t, err)
 
 			ctx, cancel := context.WithCancel(context.Background())
