@@ -1,23 +1,3 @@
-/*
- *
- * k6 - a next-generation load testing tool
- * Copyright (C) 2021 Load Impact
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Affero General Public License as
- * published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
-
 package httpext
 
 import (
@@ -43,20 +23,20 @@ func BenchmarkMeasureAndEmitMetrics(b *testing.B) {
 	logger := logrus.New()
 	logger.Level = logrus.DebugLevel
 
+	registry := metrics.NewRegistry()
 	state := &lib.State{
 		Options: lib.Options{
-			RunTags:    &metrics.SampleTags{},
 			SystemTags: &metrics.DefaultSystemTagSet,
 		},
-		Samples: samples,
-		Logger:  logger,
+		BuiltinMetrics: metrics.RegisterBuiltinMetrics(registry),
+		Samples:        samples,
+		Logger:         logger,
 	}
 	t := transport{
 		state: state,
 		ctx:   ctx,
 	}
 
-	b.ResetTimer()
 	unfRequest := &unfinishedRequest{
 		tracer: &Tracer{},
 		response: &http.Response{
