@@ -18,11 +18,8 @@ import (
 // aggregated metric sample values. They are used to generate the end-of-test
 // summary and to evaluate the test thresholds.
 type MetricsEngine struct {
-	registry       *metrics.Registry
-	executionState *lib.ExecutionState
-	options        lib.Options
-	runtimeOptions lib.RuntimeOptions
-	logger         logrus.FieldLogger
+	es     *lib.ExecutionState
+	logger logrus.FieldLogger
 
 	// These can be both top-level metrics or sub-metrics
 	metricsWithThresholds []*metrics.Metric
@@ -37,21 +34,15 @@ type MetricsEngine struct {
 }
 
 // NewMetricsEngine creates a new metrics Engine with the given parameters.
-func NewMetricsEngine(
-	registry *metrics.Registry, executionState *lib.ExecutionState,
-	opts lib.Options, rtOpts lib.RuntimeOptions, logger logrus.FieldLogger,
-) (*MetricsEngine, error) {
+func NewMetricsEngine(es *lib.ExecutionState) (*MetricsEngine, error) {
 	me := &MetricsEngine{
-		registry:       registry,
-		executionState: executionState,
-		options:        opts,
-		runtimeOptions: rtOpts,
-		logger:         logger.WithField("component", "metrics-engine"),
+		es:     es,
+		logger: es.Test.Logger.WithField("component", "metrics-engine"),
 
 		ObservedMetrics: make(map[string]*metrics.Metric),
 	}
 
-	if !(me.runtimeOptions.NoSummary.Bool && me.runtimeOptions.NoThresholds.Bool) {
+	if !(me.es.Test.RuntimeOptions.NoSummary.Bool && me.es.Test.RuntimeOptions.NoThresholds.Bool) {
 		err := me.initSubMetricsAndThresholds()
 		if err != nil {
 			return nil, err
@@ -74,7 +65,7 @@ func (me *MetricsEngine) getThresholdMetricOrSubmetric(name string) (*metrics.Me
 	// TODO: replace with strings.Cut after Go 1.18
 	nameParts := strings.SplitN(name, "{", 2)
 
-	metric := me.registry.Get(nameParts[0])
+	metric := me.es.Test.Registry.Get(nameParts[0])
 	if metric == nil {
 		return nil, fmt.Errorf("metric '%s' does not exist in the script", nameParts[0])
 	}
@@ -98,15 +89,6 @@ func (me *MetricsEngine) getThresholdMetricOrSubmetric(name string) (*metrics.Me
 
 	// TODO: reword these from "will be deprecated" to "were deprecated" and
 	// maybe make them errors, not warnings, when we introduce un-indexable tags
-
-	if _, ok := sm.Tags.Get("url"); ok {
-		me.logger.Warnf("Thresholds like '%s', based on the high-cardinality 'url' metric tag, "+
-			"are deprecated and will not be supported in future k6 releases. "+
-			"To prevent breaking changes and reduce bugs, use the 'name' metric tag instead, see"+
-			"URL grouping (https://k6.io/docs/using-k6/http-requests/#url-grouping) for more information.", name,
-		)
-	}
-
 	if _, ok := sm.Tags.Get("error"); ok {
 		me.logger.Warnf("Thresholds like '%s', based on the high-cardinality 'error' metric tag, "+
 			"are deprecated and will not be supported in future k6 releases. "+
@@ -136,10 +118,10 @@ func (me *MetricsEngine) markObserved(metric *metrics.Metric) {
 }
 
 func (me *MetricsEngine) initSubMetricsAndThresholds() error {
-	for metricName, thresholds := range me.options.Thresholds {
+	for metricName, thresholds := range me.es.Test.Options.Thresholds {
 		metric, err := me.getThresholdMetricOrSubmetric(metricName)
 
-		if me.runtimeOptions.NoThresholds.Bool {
+		if me.es.Test.RuntimeOptions.NoThresholds.Bool {
 			if err != nil {
 				me.logger.WithError(err).Warnf("Invalid metric '%s' in threshold definitions", metricName)
 			}
@@ -164,7 +146,7 @@ func (me *MetricsEngine) initSubMetricsAndThresholds() error {
 
 	// TODO: refactor out of here when https://github.com/grafana/k6/issues/1321
 	// lands and there is a better way to enable a metric with tag
-	if me.options.SystemTags.Has(metrics.TagExpectedResponse) {
+	if me.es.Test.Options.SystemTags.Has(metrics.TagExpectedResponse) {
 		_, err := me.getThresholdMetricOrSubmetric("http_req_duration{expected_response:true}")
 		if err != nil {
 			return err // shouldn't happen, but ¯\_(ツ)_/¯
@@ -181,7 +163,7 @@ func (me *MetricsEngine) EvaluateThresholds(ignoreEmptySinks bool) (thresholdsTa
 	me.MetricsLock.Lock()
 	defer me.MetricsLock.Unlock()
 
-	t := me.executionState.GetCurrentTestRunDuration()
+	t := me.es.GetCurrentTestRunDuration()
 
 	for _, m := range me.metricsWithThresholds {
 		// If either the metric has no thresholds defined, or its sinks
